<?php

namespace Drupal\social_feeds_block\Controller;

use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Controller\ControllerBase;
use Drupal\Core\Url;
use EspressoDev\InstagramBasicDisplay\InstagramBasicDisplay;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\HttpFoundation\Request;

/**
 * Returns responses for social_feeds_block routes.
 *
 * @package Drupal\social_feeds_block\Controller
 */
class InstagramAuthController extends ControllerBase {

  /**
   * The current request.
   *
   * @var \Symfony\Component\HttpFoundation\Request
   */
  protected $currentRequest;

  /**
   * InstagramAuthController constructor.
   *
   * @param \Drupal\Core\Config\ConfigFactoryInterface $config_factory
   *   The configuration service.
   * @param \Symfony\Component\HttpFoundation\Request $request
   *   The current request.
   */
  public function __construct(ConfigFactoryInterface $config_factory, Request $request) {
    $this->configFactory = $config_factory;
    $this->currentRequest = $request;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('config.factory'),
      $container->get('request_stack')->getCurrentRequest()
    );
  }

  /**
   * Get an Instagram access token.
   */
  public function accessToken() {
    $code = $this->currentRequest->query->get('code');
    $token = '';
    if ($code) {
      $config = $this->configFactory->getEditable('config.instagram_social_feeds_block');

      $instagram = new InstagramBasicDisplay([
        'appId' => $config->get('insta_app_id'),
        'appSecret' => $config->get('insta_secret'),
        'redirectUri' => Url::fromRoute('social_feeds_block.instagram_auth', [], ['absolute' => TRUE])->toString(),
      ]);

      // Get the short-lived access token (valid for 1 hour)
      $token = $instagram->getOAuthToken($code, TRUE);

      // Exchange this token for a long lived token (valid for 60 days)
      if ($token) {
        $token = $instagram->getLongLivedToken($token, TRUE);
        $config->set('insta_access_token', $token);
        $config->set('insta_access_token_date', time());
        $config->save();
      }
    }

    return [
      '#markup' => $this->t('Your Access Token has been generated and saved. %token', ['%token' => $token]),
    ];
  }

}
